/* bench.c */

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "crde.h"

#define NORM	0x07
#define HI	0x0F
#define INV	0x70

TABLE *sups, *parts, *orders, *answer;
clock_t t1, t2;
long result;

char *cities[20] = {
  "Atlanta",
  "Augusta",
  "Boston",
  "Chicago",
  "Daytona",
  "Denver",
  "Detroit",
  "Hong Kong",
  "Houston",
  "London",
  "Los Angeles",
  "Munich",
  "New York",
  "Nashville",
  "Palm Beach",
  "Rochester",
  "San Francisco",
  "Singapore",
  "Tokyo",
  "Washington, D.C.",
};

char *names[20] = {
  "Anderson, Mark",
  "Barnes, Edward",
  "Cutler, Cheryl",
  "Cool, Joe",
  "Denison, Susan",
  "Ferris, Kaye",
  "Huddle, Jean",
  "Kahn, Emily",
  "Knight, John",
  "Morse, Timothy",
  "Miller, James",
  "McCaffrey, Timothy",
  "Nesmith, Cory",
  "Owosso, Peter",
  "Ricter, Pamela",
  "Sheldon, Al",
  "Thomson, Amy",
  "Treat, Bill",
  "Young, Bob",
  "Zywicki, Dave"
};


/* utility routines */

void wait()
{
  textattr(INV + 0x80);
  gotoxy(1, 25); clreol(); cprintf("Hit any key to continue...");
  textattr(NORM);
  getch();
}

void disptime()
{
  double d;
  long l;

  textattr(INV);
  gotoxy(1, 25);
  d = t2 - t1;
  d /= CLK_TCK;
  cprintf("Time elapsed = %lf seconds.", d);
  for (l = 0; l < 500000; l++);
}

void bench_start(char *title, char *des)
{
  textattr(NORM);
  clrscr();
  textattr(INV);
  gotoxy(1,  1); clreol(); cputs(title);
  gotoxy(1,  2); clreol(); cputs(des);
  textattr(INV + 0x80);
  gotoxy(1, 25); clreol(); cputs("Press any key to begin operation...");
  getch();
  gotoxy(1, 25); clreol(); cputs("Working...");
  textattr(NORM);
  t1 = clock();
}

void bench_end(void)
{
  t2 = clock();
  disptime();
}

int show_table(TABLE *answer, char *des)
{
  int i;

  textattr(NORM);
  clrscr();
  textattr(INV);
  gotoxy(1,  1); clreol(); cprintf("Viewing %s table.", des);
  gotoxy(1,  2); clreol(); cprintf("%ld rows found.", trows(answer));
  textattr(INV + 0x80);
  gotoxy(1, 25); clreol(); cprintf("Press [RETURN] or [ESC] to continue...");
  i = tview(answer, NULL, NULL, 3, 5, 78, 23, NORM, NORM, HI, NULL);
  textattr(INV);
  gotoxy(1, 25); clreol();
  textattr(NORM);
  return i;
}


/* init routine */

int test_init()
{
  FILE *f;
  int i;

  f = fopen("sups.txt", "w");
  for (i = 0; i < 1000; i++)
    fprintf(f, "%d,\"%s\",\"%s\",%d\n", i, names[random(20)], cities[random(20)], random(30));
  fclose(f);

  f = fopen("orders.txt", "w");
  for (i = 0; i < 5000; i++)
    fprintf(f, "%d,%d,%d\n", random(1000), random(1000), random(50) * 10);
  fclose(f);
  return 0;
}


/* benchmarks */

int test_import()
{
  bench_start("Import Test", "Import 1000 row and 5000 row keyed tables.");
  sups = timportascii("sups", "*s# i, sname c26, city c26, status i",
    "sups.txt");
  orders = timportascii("orders", "*s# i, *p# i, qty i", "orders.txt");
  bench_end();
  show_table(sups, "SUPPLIERS");
  show_table(orders, "ORDERS");
  wait();
  return 0;
}

int test_select1()
{
  bench_start("Indexed Select", "Select all rows matching an unindexed criteria");
  answer = tselect(STATIC, sups, "s#", EQ, 100, NULL);
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_select2()
{
  bench_start("Un-Indexed Select", "Select all rows matching an indexed criteria");
  answer = tselect(STATIC, sups, "city", EQ, "Boston", NULL);
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_2join()
{
  bench_start("2-Table Join", "Join 1000x5000 row tables (don't try this in dBASE!).");
  answer = tjoin(STATIC, sups, "s#", orders, "s#", "s#, sname, p#, qty");
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_index()
{
  bench_start("Secondary Index", "Generate a Secondary index on a 1000 row table.");
  result = tindex(sups, "city");
  bench_end();
  wait();
  return 0;
}

int test_project()
{
  bench_start("Distinct Projection", "Project all distinct values from one column in 1000 row table.");
  answer = tproject(STATIC, sups, "*city", NULL);
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_sort()
{
  bench_start("Sort", "Sort a 1000 sups table by city.");
  answer = tsort(STATIC, sups, "city");
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_change()
{
  bench_start("Update Test", "Change all rows containg 'Boston' to 'BOSTON'");
  result = tchange(sups, "city", "BOSTON", NULL, "city", EQ, "Boston", NULL);
  bench_end();
  show_table(sups, "SUPPLIER");
  wait();
  return 0;
}

int test_restruct()
{
  bench_start("Restructure Test", "Remove one column form a table and shorten another.");
  answer = trestruct(STATIC, sups, "*s# i, sname c26, city c16");
  bench_end();
  show_table(answer, "ANSWER");
  tdrop(answer);
  wait();
  return 0;
}

int test_delete()
{
  bench_start("Delete Test", "Delete all rows matching a specific criteria.");
  tdelete(sups, "status", GT, 25, NULL);
  bench_end();
  show_table(sups, "SUPPLIER");
  wait();
  return 0;
}

int main()
{
  /* init crde */
  if (tinit(NET_NONE, NULL, 64)) {
    printf("error:  unable to initialize CRDE 2.0.\n");
    exit(1);
  }

  /* generate dummy ascii files */
  textattr(NORM);
  clrscr();
  printf("Generating ascii files used by benchmark program.\n");
  test_init();

  /* benchmarks */
  test_import();
  test_select1();
  test_select2();
  test_2join();
  test_index();
  test_project();
  test_sort();
  test_change();
  test_restruct();
  test_delete();

  clrscr();
  printf("CRDE benchmark program completed!\n");
  tquit();
  return 0;
}

/* end of bench.c */

